@@ -3,6 +3,7 @@ extern crate derive_more;
 
 use chrono::NaiveDateTime;
 use sqlx::{error::Error as DBError, Pool, Sqlite, SqlitePool};
+use std::num::NonZeroU64;
 use std::path::PathBuf;
 use std::sync::{Arc, RwLock};
 
@@ -11,6 +12,8 @@ mod event;
 pub mod migration;
 pub use event::*;
 
+const ZERO: i64 = 0;
+
 #[derive(Debug, Clone)]
 pub struct Historian {
     pool: Arc<RwLock<SqlitePool>>,
@@ -57,7 +60,7 @@ impl<'a> DBEvent<'a> {
             time,
             ty,
             cmd,
-            main_event_id: 0,
+            main_event_id: ZERO,
             content: None,
             args: None,
         }
@@ -76,8 +79,65 @@ impl<'a> DBEvent<'a> {
     }
 }
 
+macro_rules! select_last_arg {
+    ($select:literal, $script_id:expr, $offset:expr, $limit:expr) => {{
+        static EXEC_TY: &'static str = EventType::Exec.get_str();
+        sqlx::query!(
+            "
+            WITH args AS (
+                SELECT args, max(time) as time FROM events
+                WHERE type = ? AND script_id = ? AND NOT ignored
+                GROUP BY args
+                ORDER BY time DESC LIMIT ? OFFSET ?
+            ) SELECT "
+                + $select
+                + " FROM args
+            ",
+            EXEC_TY,
+            $script_id,
+            $limit,
+            $offset
+        )
+    }};
+}
+
+macro_rules! ignore_arg {
+    ($pool:expr, $cond:literal, $($var:expr),+) => {
+        let exec_ty = EventType::Exec.get_str();
+        let done_ty = EventType::ExecDone.get_str();
+        sqlx::query!(
+            "
+            UPDATE events SET ignored = true
+            WHERE type = ? AND
+            "
+                + $cond,
+            exec_ty,
+            $($var),*
+        )
+        .execute(&*$pool)
+        .await?;
+
+        sqlx::query!(
+            "
+            UPDATE events SET ignored = true
+            WHERE type = ? AND main_event_id IN (
+                SELECT id FROM events WHERE type = ? AND "
+                + $cond
+                + "
+            )
+            ",
+            done_ty,
+            exec_ty,
+            $($var),*
+        )
+        .execute(&*$pool)
+        .await?;
+    };
+}
+
 #[derive(Debug)]
 pub struct IgnoreResult {
+    pub script_id: i64,
     pub exec_time: Option<NaiveDateTime>,
     pub exec_done_time: Option<NaiveDateTime>,
 }
@@ -111,7 +171,7 @@ impl Historian {
 
     pub async fn record(&self, event: &Event<'_>) -> Result<i64, DBError> {
         log::debug!("記錄事件 {:?}", event);
-        let ty = event.data.get_type().to_string();
+        let ty = event.data.get_type().get_str();
         let time = event.time;
         let cmd = std::env::args().collect::<Vec<_>>().join(" ");
         let mut db_event = DBEvent::new(event.script_id, time, &ty, &cmd);
@@ -143,7 +203,7 @@ impl Historian {
                 code,
                 main_event_id,
             } => {
-                let exec_ty = EventType::Exec.to_string();
+                let exec_ty = EventType::Exec.get_str();
                 let ignored_res = sqlx::query!(
                     "SELECT ignored FROM events WHERE type = ? AND id = ?",
                     exec_ty,
@@ -152,7 +212,7 @@ impl Historian {
                 .fetch_one(&*self.pool.read().unwrap())
                 .await?;
                 if ignored_res.ignored {
-                    return Ok(0);
+                    return Ok(ZERO);
                 }
                 let code = code.to_string();
                 self.raw_record(db_event.content(&code).main_event_id(*main_event_id))
@@ -163,7 +223,7 @@ impl Historian {
     }
 
     pub async fn last_args(&self, id: i64) -> Result<Option<String>, DBError> {
-        let ty = EventType::Exec.to_string();
+        let ty = EventType::Exec.get_str();
         let res = sqlx::query!(
             "
             SELECT args FROM events
@@ -186,97 +246,130 @@ impl Historian {
     ) -> Result<impl ExactSizeIterator<Item = String>, DBError> {
         let limit = limit as i64;
         let offset = offset as i64;
-        let ty = EventType::Exec.to_string();
-        let res = sqlx::query!(
-            "
-            WITH args AS (
-                SELECT args, max(time) as time FROM events
-                WHERE type = ? AND script_id = ? AND NOT ignored
-                GROUP BY args
-                ORDER BY time DESC LIMIT ? OFFSET ?
-            ) SELECT args FROM args
-            ",
-            ty,
-            id,
-            limit,
-            offset
-        )
-        .fetch_all(&*self.pool.read().unwrap())
-        .await?;
+        let res = select_last_arg!("args", id, offset, limit)
+            .fetch_all(&*self.pool.read().unwrap())
+            .await?;
         Ok(res.into_iter().map(|res| res.args.unwrap_or_default()))
     }
 
-    pub async fn ignore_args(
-        &self,
-        script_id: i64,
-        number: std::num::NonZeroU64,
-    ) -> Result<Option<IgnoreResult>, DBError> {
-        let number = number.get();
-        let exec_ty = EventType::Exec.to_string();
-        let done_ty = EventType::ExecDone.to_string();
-        let offset = number as i64 - 1;
+    async fn make_ignore_result(&self, script_id: i64) -> Result<IgnoreResult, DBError> {
+        Ok(IgnoreResult {
+            script_id,
+            exec_time: self.last_time_of(script_id, EventType::Exec).await?,
+            exec_done_time: self.last_time_of(script_id, EventType::ExecDone).await?,
+        })
+    }
+    pub async fn ignore_args_by_id(&self, event_id: i64) -> Result<Option<IgnoreResult>, DBError> {
+        if event_id == ZERO {
+            log::info!("試圖忽略零事件，什麼都不做");
+            return Ok(None);
+        }
+
         let pool = self.pool.read().unwrap();
-        let args = sqlx::query!(
+        let exec_ty = EventType::Exec.get_str();
+        let latest_record = sqlx::query!(
             "
-            WITH args AS (
-                SELECT args, max(time) as time FROM events
-                WHERE type = ? AND script_id = ? AND NOT ignored
-                GROUP BY args
-                ORDER BY time DESC LIMIT 1 OFFSET ?
-            ) SELECT args FROM args
+            SELECT id, script_id FROM events
+            WHERE type= ? AND script_id = (SELECT script_id FROM events WHERE id = ?)
+            ORDER BY time DESC LIMIT 1
             ",
             exec_ty,
-            script_id,
-            offset,
+            event_id,
         )
         .fetch_one(&*pool)
-        .await?
-        .args;
-
-        sqlx::query!(
-            "
-            UPDATE events SET ignored = true
-            WHERE script_id = ? AND type = ? AND args = ?
-            ",
-            script_id,
-            exec_ty,
-            args,
-        )
-        .execute(&*pool)
         .await?;
+        // TODO: check if this event is exec?
 
-        sqlx::query!(
-            "
-            UPDATE events SET ignored = true
-            WHERE script_id = ? AND type = ? AND main_event_id IN (
-                SELECT id FROM events WHERE args = ? AND script_id = ?
-            )
-            ",
-            script_id,
-            done_ty,
-            args,
-            script_id,
-        )
-        .execute(&*pool)
-        .await?;
+        ignore_arg!(pool, "id = ?", event_id);
 
-        if number == 1 {
+        if latest_record.id == event_id {
             log::info!("ignore last args");
-            let ret = IgnoreResult {
-                exec_time: self.last_time_of(script_id, EventType::Exec).await?,
-                exec_done_time: self.last_time_of(script_id, EventType::ExecDone).await?,
-            };
+            let ret = self.make_ignore_result(latest_record.script_id).await?;
             return Ok(Some(ret));
         }
         Ok(None)
     }
+    pub async fn ignore_args_range(
+        &self,
+        script_id: i64,
+        min: NonZeroU64,
+        max: Option<NonZeroU64>,
+    ) -> Result<Option<IgnoreResult>, DBError> {
+        let offset = min.get() as i64 - 1;
+        let limit = if let Some(max) = max {
+            (max.get() - min.get()) as i64
+        } else {
+            -1
+        };
+
+        let pool = self.pool.read().unwrap();
+        let args_vec = select_last_arg!("args", script_id, offset, limit)
+            .fetch_all(&*pool)
+            .await?;
+
+        for args in args_vec {
+            // TODO: 有沒有更有效的方法？
+            ignore_arg!(pool, "script_id = ? AND args = ?", script_id, args.args);
+        }
+
+        if offset == 0 {
+            log::info!("ignore last args");
+            let ret = self.make_ignore_result(script_id).await?;
+            return Ok(Some(ret));
+        }
+        Ok(None)
+    }
+    pub async fn ignore_args(
+        &self,
+        script_id: i64,
+        number: NonZeroU64,
+    ) -> Result<Option<IgnoreResult>, DBError> {
+        let number = number.get();
+        let offset = number as i64 - 1;
+        let pool = self.pool.read().unwrap();
+
+        let args = select_last_arg!("args", script_id, offset, 1)
+            .fetch_one(&*pool)
+            .await?
+            .args;
+
+        ignore_arg!(pool, "script_id = ? AND args = ?", script_id, args);
+
+        if offset == 0 {
+            log::info!("ignore last args");
+            let ret = self.make_ignore_result(script_id).await?;
+            return Ok(Some(ret));
+        }
+        Ok(None)
+    }
+
+    pub async fn amend_args_by_id(&self, event_id: i64, args: &str) -> Result<(), DBError> {
+        if event_id == ZERO {
+            log::info!("試圖修改零事件，什麼都不做");
+            return Ok(());
+        }
+
+        let exec_ty = EventType::Exec.get_str();
+        sqlx::query!(
+            "
+            UPDATE events SET ignored = false, args = ?
+            WHERE type = ? AND id = ?
+            ",
+            args,
+            exec_ty,
+            event_id
+        )
+        .execute(&*self.pool.read().unwrap())
+        .await?;
+        Ok(())
+    }
 
     pub async fn last_time_of(
         &self,
         script_id: i64,
         ty: EventType,
     ) -> Result<Option<NaiveDateTime>, DBError> {
-        let ty = ty.to_string();
+        let ty = ty.get_str();
         let time = sqlx::query!(
             "
             SELECT time FROM events
@@ -293,7 +386,7 @@ impl Historian {
 
     pub async fn tidy(&self, script_id: i64) -> Result<(), DBError> {
         let pool = self.pool.read().unwrap();
-        let exec_ty = EventType::Exec.to_string();
+        let exec_ty = EventType::Exec.get_str();
         // XXX: 笑死這啥鬼
         sqlx::query!(
             "
